/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: ISR.c
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/


#include <avr/io.h>
#include <avr/interrupt.h>

#include "ui.h"
#include "scheduler.h"
#include "dynwatches.h"
#include "system.h"

#include "string.h"
#include "driver\uart\uart.h"
#include "libinterface.h"
#include "namesaliases.h"
#include "driver/pwm/libPWM.h"

#include "debug.h"

ISR(INT1_vect){

    uint8_t key = 0;

    cli();

    if( PIND & (1 << 3) ){
        return;
    }

    if( PIND & (1 << 4) ){
        key |= (1<<0);
        ui_rotations++;
    }else{
        key |= (1<<1);
        ui_rotations--;
    }

    sei();

    if( !(PIND & (1 << 2)) ){
        key |= (1<<2);
    }

    ui_KeyIn(key);
}

ISR(INT0_vect){

    //Crashes when using rotary encoder was found to be located in dynamic watch library
    //-> possible dynamic memory fail

    uint8_t key = 0;

    if(! (PIND & (1 << 2)) ){
        key |= (1<<2);

        sh_new_appointment(sh_app_once, shui_longkey, ui_KeyLong, ui_longtime, 0);
    }else{
        key |= (1<<3);
    }

    ui_KeyIn(key);
}

ISR(TIMER0_OVF_vect){
    PWM_ISR();
}


ISR(TIMER1_OVF_vect){
    TCNT1 = 62411;

    LedOn();

    ui_uptime++;
    sh_timer();
    w_tick_watch();

    LedOff();

    //add 10ms code here

}

ISR(BADISR_vect) {
    debugIfOut(4, 0, "Bad ISR VECT");
}
