/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: UiFuncs.c
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <inttypes.h>

#include "UiFuncs.h"
#include "libInput.h"
#include "string.h"

#include "debug.h"

#include "ui.h"
#include "system.h"
#include "libinterface.h"
#include "namesaliases.h"
#include "settings.h"

#include "displaymanagement.h"
#include "menu.h"
#include "menuhandlers.h"


uint16_t temp = 0;
int8_t  pidIn = 0;
uint8_t pidinstat = 0;
uint8_t uifSysScreen = 0;

void SetupHeaterMenu(uint8_t HeatID);

menudata heatermenu[7] = {
    {menuCnull, 0, "Heater setup", 3},
    {mh_ChSetpnt, 0, "Set Temp.", 0},
    {menuCnull, 0, "PID config", 3},
    {mh_BackSysScr, 0, "OK", 0},
    {mh_ChP, 0, "Set P share", 0},
    {mh_ChI, 0, "Set I share", 0},
    {mh_ChD, 0, "Set D share", 0}
    };

menudata mainmenu[2] = {
    {menuCnull, 0, "Menu", 1},
    {mm_facRes, 0, "factory reset", 0}
    };

/// Screen data generator functions

uint8_t uifSysView(char *cout, uint8_t *len, uint8_t lcdsize, uint8_t key){
    char cNum[10];
    char cNewline[2] = {(char) 10, (char) 0};
    uint16_t temp = 0;
    uint8_t pwm = 0;

    switch(key){
        case uik_longdn:
            sys_deactivate(uifSysScreen);
            SetupHeaterMenu(uifSysScreen);
            dm_changeSource(uifHeaterMenu);
        break;

        case uik_longlongdn:
            sys_deactivate(uifSysScreen);
            init_menu(mainmenu, 2);
            dm_changeSource(uifMainMenu);
        break;

        case uik_klick:
            if(Descriptors[uifSysScreen].status == 0){
                sys_deactivate(uifSysScreen);
            }else{
                sys_activate(uifSysScreen);
            }
        break;

        case uik_left:
            uifSysScreen++;
            if(uifSysScreen == regulator_amount){
                uifSysScreen = 0;
            }
        break;

        case uik_right:
            uifSysScreen--;
            if(uifSysScreen == 255){
                uifSysScreen = regulator_amount -1;
            }
        break;
    }

    if(cout == 0){
        return 0;
    }

    str_cpy_p(cout, "Pg");

    inttostr8(cNum, uifSysScreen, 1);
    str_cat(cout, cNum);
    str_cat_p(cNum, "     ");

    if(Descriptors[uifSysScreen].status != 0){
        switch(Descriptors[uifSysScreen].status){

            case ThermErr_TcOpen:
                str_cat_p(cout, " OFF TC");
            break;

            case ThermErr_UserOff:
                str_cat_p(cout, " OFF USER");
            break;

            case ThermErr_OffNoErr:
                str_cat_p(cout, " OFF NOERR");
            break;

            default:
                str_cat_p(cout, " OFF (?)");
            break;
        }

    }else{
        str_cat_p(cout, " T: ");
        ifRead(d_max, Descriptors[uifSysScreen].therm_adr, if_TODTP &temp, 2);
        uiTempstr(cNum, temp);
        str_cat(cout, cNum);
    }

    str_cat(cout, cNewline);

    str_cat_p(cout, "S ");

    temp = (uint16_t) ((Descriptors[uifSysScreen].regulator)->setpoint);
    uiTempstr(cNum, temp);
    str_cat(cout, cNum);

    str_cat_p(cout, " PWM ");

    ifRead(d_pwm, Descriptors[uifSysScreen].PWM_adr, if_TODTP &pwm, 1);
    inttostr8(cNum, pwm, 3);
    str_cat(cout, cNum);

    *len = str_len(cout);

    return 0;
}


uint8_t uifHeaterMenu(char *cout, uint8_t *len, uint8_t lcdsize, uint8_t key){

    uint8_t i = 0;

    mStatemachine(key);

    if(cout != 0){
        mGetText(cout, 2, 16, 0);

        ///match for dual line support
        for( i = 0 ; i < 20 ; i++){
            if( cout[i] == (char) 0){
                cout[i] = 10;
            }
        }

        *len = str_len(cout);
    }

    return 0;
}

uint8_t uifMainMenu(char *cout, uint8_t *len, uint8_t lcdsize, uint8_t key){
    uint8_t i = 0;

    mStatemachine(key);

    if(cout != 0){
        mGetText(cout, 2, 16, 0);

        ///match for dual line support
        for( i = 0 ; i < 20 ; i++){
            if( cout[i] == (char) 0){
                cout[i] = 10;
            }
        }

        *len = str_len(cout);
    }

    return 0;
}

uint8_t uifSettemp(char *cout, uint8_t *len, uint8_t lcdsize, uint8_t key){
    uint8_t ret = 0;

    ret = inUI16(cout, len, "Set Temp:", "C", key, &temp, 4, 1, 40);

    if(ret == inR_Complete){
        (Descriptors[uifSysScreen].regulator)->setpoint = temp;
        store_word_setting(&(set_HeatSetpoints[uifSysScreen]), temp);
        temp = 0;

        dm_changeSource(uifSysView);
    }

    return 0;
}

uint8_t uifSetPid(char *cout, uint8_t *len, uint8_t lcdsize, uint8_t key){
    uint8_t ret = 0;

    ret = inI8(cout, len, "Set PID part:", "%", key, &pidIn, 1, 1, 40);

    if(ret == inR_Complete){
        switch(pidinstat){
            case 1:
                (Descriptors[uifSysScreen].regulator)->pPart = pidIn;
            break;
            case 2:
                (Descriptors[uifSysScreen].regulator)->iPart = pidIn;
            break;
            case 3:
                (Descriptors[uifSysScreen].regulator)->dPart = pidIn;
            break;
        }
        store_byte_setting(&(set_PidParams[uifSysScreen][pidinstat -1]), pidIn);
        pidIn = 0;

        dm_changeSource(uifSysView);
    }

    return 0;
}


/// Helper functions

void SetupHeaterMenu(uint8_t HeatID){
    uint8_t i = 0;

    for( i = 0 ; i < 6 ; i++){
        heatermenu[i].CallBackArgument = HeatID;
    }

    init_menu(heatermenu, 7);
}

uint8_t uiTempstr(char *cout, uint16_t temp){

    char cNumbers[21];
    uint16_t hilf = 0;
    uint8_t NumLen = 0, len = 0;


    if(cout == 0){
        return 0;
    }


    hilf = temp / 4;
    NumLen = inRlg10(hilf);
    NumLen += 1;
    inttostr16(cNumbers, hilf, NumLen);

    str_cpy(cout, cNumbers);
    len += str_len(cNumbers);

    str_cat_p(cout, ".");

    hilf = temp % 4;
    hilf *= 100;
    hilf /= 4;
    NumLen = inRlg10(hilf);
    NumLen += 1;
    inttostr16(cNumbers, hilf, NumLen);

    str_cat(cout, cNumbers);
    len += str_len(cNumbers);

    len += 1;

    return 0;
}

