
/*
-------------------------------------------------------
    Interface - A simple hardware abstraction library

    by Tobias Schlegel - www.tobias-schlegel.de
-------------------------------------------------------

Copyright (C) 2010  Tobias Schlegel,
                    tobias@drschlegel.de
                    www.tobias-schlegel.de

Report bugs to:     tobias@drschlegel.de

This program is free software; you can redistribute it and/or modify it under the terms of the GNU
General Public License as published by the Free Software Foundation; either version 3 of the License,
or any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/

/*
This is an excample of a standard driver.
*/

#include <avr/io.h>

#include "../../libinterface.h"

#include "lcd_driver.h"

#include "lcd.h"

/**
    Write function...
    @param[in]      adr     Address offset for interface (can be fixed, depends on IF-config)
    @param[in,out]  * data  Pointer to data to write. can be used as two-way communication
    @param[in]      len     Amount of units to write (in n * if_datatype)
*/
uint8_t lcd_write(uint16_t address, if_datatype * data, uint8_t len){

    uint8_t i = 0, crd[2];

    switch(address){

        default:
        case 0: //write
            for(i = 0 ; i < len ; i++){

                switch (data[i]){

                    case 0:
                        return if_OK;
                    break;

                    default:
                        lcd_putc(data[i]);
                    break;
                }
            }
        break;

        case 1: //writeln

            lcd_control(lcdc_clear, if_TODTP 0);
            crd[0] = 0;
            crd[1] = 0;

            for(i = 0 ; i < len ; i++){

                switch (data[i]){

                    case 0:
                        return if_OK;
                    break;

                    case 10:
                        crd[1]++;
                        lcd_control(lcdc_goto, if_TODTP crd);
                    break;

                    default:
                        lcd_putc(data[i]);
                    break;
                }
            }
        break;
    }

    return if_OK;
}


/**
    No use at the moment
    @param[in]      adr     Address offset for interface (can be fixed, depends on IF-config)
    @param[in,out]  * data  Pointer to data to read. can be used as two-way communication
    @param[in]      len     Amount of units to write (in n * if_datatype)
*/
uint8_t lcd_read(uint16_t address, if_datatype * data, uint8_t len){

    return if_ERROR;
}


/**
    Control function...
    @param[in]      control Control-Word
    @param[in,out]  * data  Pointer to data to read/write. can be used as two-way communication
*/
uint8_t lcd_control(uint16_t control, if_datatype * data){

    uint8_t hi = 0;

    switch(control){

        case lcdc_shutdown:
            lcd_init(LCD_DISP_OFF);
        break;

        case lcdc_init:
            if(data[0] == 0){
                hi = LCD_DISP_ON;
            }else{
                hi = data[0];
            }

            lcd_init(hi);
            lcd_clrscr();
        break;

        case lcdc_clear:
            lcd_clrscr();
        break;

        case lcdc_goto:
            lcd_gotoxy( data[0], data[1]);
        break;

        default:
            return if_CONTROL_UNKNOWN;
        break;
    }

    return if_OK;
}

