
/*
*   MAX6675 multichip library
*   Read data from a MAX6675 thermocouple IC via software defined routines.
*   Version for multiple chips.
*
*   File: max6675_m.c
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/


#include <avr/io.h>
#include "max6675_m.h"

uint8_t                 CS_NR[chips_connected]     = {3, 2};
volatile uint8_t *      CS_PORT[chips_connected]   = {&PORTA, &PORTA};
volatile uint8_t *      CS_DDR[chips_connected]    = {&DDRA, &DDRA};


//gettemp () returns absolute Temperature in Temp * 4 C - in 1/4-steps
//
uint16_t gettemp(uint8_t chip ){													//Temperatur holen.
	uint8_t bit = 0, bitnr = 12;										//Variablen
	uint8_t foo1 = 0;
	uint16_t Rohdata = 0;



	*(CS_PORT[chip]) &= ~(1 << CS_NR[chip]);												//Chip select anlegen

	for(foo1 = 0 ; foo1 < 16 ; foo1++){									//16 Bits einlesen

		bit = 15 - foo1;												//Die Aktuelle Bitnr berechnen.


		SCK_Port |= (1 << SCK);											//SCK hi


		if((bit <= 14) && (bit >= 3)){									//Einfach mal die 12 relevanten von den 16 Bits ausfiltern

			if((SO_Pin & (1 << SO))){										//WENN SO 1 ist, dann...
				bitnr--;												//zhlen wir runter...
				Rohdata |= (1 << bitnr);								// und schieben eine 1 an bit x
			}else{														//WENN dem NICHT so ist, dann...
				bitnr--;												//zhlen wir runter...
				Rohdata &= ~(1 << bitnr);								//und schieben eine 0 an bit x
			}

		}else{															//weis au nimmer, was das soll.
			bitnr = 12;
		}

		SCK_Port &= ~(1 << SCK);										//SCK LO
	}


	*(CS_PORT[chip]) |= (1 << CS_NR[chip]);												//CS HI	//Alles auf Standardkonfig.


	return Rohdata;														//Das ist doch mal was ;D
}


// getTC() returns 0 if Thermocouple is not connected, 255 if thermocuple is connected
// (to enable this feature T- must be connected to GND)
uint8_t getTC(uint8_t chip){													//Temperatur holen.										//Variablen
	uint8_t TC = 0;
	uint8_t	foo1 = 0;

	*(CS_PORT[chip]) &= ~(1 << CS_NR[chip]);												//Chip select anlegen

	for(foo1 = 0 ; foo1 < 16 ; foo1++){									//16 Bits einlesen												//Die Aktuelle Bitnr berechnen.

		SCK_Port |= (1 << SCK);											//SCK hi

		if(foo1 == 2){													//das 3. bit ist fr uns relevant.

			if((SO_Pin & (1 << SO))){										//WENN SO 1 ist, dann...
				TC = 0;
			}else{														//WENN dem NICHT so ist, dann...
				TC = 255;
			}
		}

		SCK_Port &= ~(1 << SCK);										//SCK LO
	}

	*(CS_PORT[chip]) |= (1 << CS_NR[chip]);												//CS HI	//Alles auf Standardkonfig.

	return TC;															//Das ist doch mal was ;D
}




//Initiates the MAX6675 and IO-pins
void init_6675(void){													//AVR initialisieren

    uint8_t i = 0;

    for(i = 0 ; i < chips_connected ; i++){
        *(CS_DDR[i]) |= (1 << CS_NR[i]);
        *(CS_PORT[i]) |= (1 << CS_NR[i]);								//CS HI	//Alles auf Standardkonfig.
    }

	SO_DDR &= ~(1 << SO);
	SO_Port |= (1<<SO);													//Pullups an. (Wichtig fr MAX6675, der kann nicht anders, hab ich festgestellt...

	SCK_DDR |= (1 << SCK);												//IOs setzen
	SCK_Port &= ~(1 << SCK);											//SCK LO


}

