
/*
-------------------------------------------------------
    Interface - A simple hardware abstraction library

    by Tobias Schlegel - www.tobias-schlegel.de
-------------------------------------------------------

Copyright (C) 2010  Tobias Schlegel,
                    tobias@drschlegel.de
                    www.tobias-schlegel.de

Report bugs to:     tobias@drschlegel.de

This program is free software; you can redistribute it and/or modify it under the terms of the GNU
General Public License as published by the Free Software Foundation; either version 3 of the License,
or any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/

/*
This is an excample of a standard driver.
*/

#include <avr/io.h>

#include "../../libinterface.h"

#include "max_driver.h"

#include "max6675_m.h"

volatile uint16_t   max_temp[chips_connected];
volatile uint8_t    max_TC[chips_connected];

/**
    Read function...
    @param[in]      adr     Address offset for interface (can be fixed, depends on IF-config)
    @param[in,out]  * data  Pointer to data to read. can be used as two-way communication
    @param[in]      len     Amount of units to write (in n * if_datatype)
*/
uint8_t max_read(uint16_t address, if_datatype * data, uint8_t len){

    uint8_t hi = 0, lo = 0;

    if(len == 0){
        return if_ERROR;
    }

    hi = (uint8_t)(uint16_t)((address  & 0xFF00) >> 8);
    lo = (uint8_t)(address  & 0x00FF);

    switch(hi){
        default:
            data[1] = (uint8_t)(uint16_t)((max_temp[lo] & 0xFF00) >> 8);
            data[0] = (uint8_t)(uint16_t)(max_temp[lo] & 0xFF);
        break;

        case 1:
            data[0] = max_TC[lo];
        break;
    }

    return if_OK;
}


/**
    Control function...
    @param[in]      control Control-Word
    @param[in,out]  * data  Pointer to data to read/write. can be used as two-way communication
*/
uint8_t max_control(uint16_t control, if_datatype * data){
    uint8_t i = 0;

    switch(control){
        case max_init:
            init_6675();
        break;

        case max_measure:
            for(i = 0 ; i < chips_connected ; i++){
                max_temp[i] = gettemp(i);
                max_TC[i] = getTC(i);
            }
        break;

        default:
            return if_CONTROL_UNKNOWN;
        break;
    }

    return if_OK;
}

