/*
*   LibInterface
*   A very basic hardware abstraction library.
*   -> Default driver template
*
*   File: PWMdriver.c
*
*   Version:    1.0
*   File-Ver.:  1.0 (05.Jan.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       5.Jan.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <avr/io.h>

#include "../../libinterface.h"
#include "PWMdriver.h"

#include "../../debug.h"

#include "libPWM.h"

/**
    Write function...
    @param[in]      adr     Address offset for interface (can be fixed, depends on IF-config)
    @param[in,out]  * data  Pointer to data to write. can be used as two-way communication
    @param[in]      len     Amount of units to write (in n * if_datatype)
*/
uint8_t pwm_write(uint16_t address, if_datatype * data, uint8_t len){
    uint8_t i = 0;

    if(len > (PWMchannels - address)){
        return if_ERROR;
    }

    for(i = 0 ; i < len ; i++){
        PWM_SetValue(address + i, data[0]);
    }


    return if_OK;
}


/**
    Read function...
    @param[in]      adr     Address offset for interface (can be fixed, depends on IF-config)
    @param[in,out]  * data  Pointer to data to read. can be used as two-way communication
    @param[in]      len     Amount of units to write (in n * if_datatype)
*/
extern volatile uint8_t PWM[];
uint8_t pwm_read(uint16_t address, if_datatype * data, uint8_t len){

    if(address < PWMchannels){
        data[0] = PWM[address];
    }

    return if_OK;
}


/**
    Control function...
    @param[in]      control Control-Word
    @param[in,out]  * data  Pointer to data to read/write. can be used as two-way communication
*/
uint8_t pwm_control(uint16_t control, if_datatype * data){

    switch(control){

        case defaultc_shutdown:
        break;

        case defaultc_init:
            init_PWM();
        break;


        default:
            return if_CONTROL_UNKNOWN;
        break;
    }

    return if_OK;
}


