
/*
*   libPWM
*   Simple multichannel 8Bit Soft-PWM library
*
*   File: libPWM.c
*
*   Version:    1.0beta
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       5.Jan.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <avr/io.h>
#include "libPWM.h"

uint8_t organizepwm(uint8_t * pw, uint8_t ** point);
void SetActivePwmChannels(void);
void pwmpinoff(uint8_t * pin);
void startPwmCycle(void);

volatile uint8_t            PWMstate = 0;
volatile uint16_t           pwmvalue = 0;
volatile uint8_t            PWM[PWMchannels];
volatile uint8_t            PwmLoad[PWMchannels];
volatile uint8_t            *PWMsort[PWMchannels];
volatile uint8_t            *PWMports[PWMchannels] = PWMprts;
volatile uint8_t            PWMpins[PWMchannels] = PWMpns;

void init_PWM(void){
    uint8_t i = 0;

    for(i = 0 ; i < PWMchannels ; i++){
        PWM[i] = 0;
        PwmLoad[i] = 0;
    }
}

void PWM_SetValue(uint8_t channel, uint8_t value){
    if(channel < PWMchannels){
        PwmLoad[channel] = value;
    }
}

void PWM_ISR(void){
    uint8_t use = 0, hlp = 0;

    if(PWMstate == 0){
        startPwmCycle();

        PwmTimerCountRegister = (uint8_t)(uint16_t) (0xFF - *(PWMsort[PWMstate -1]));
        pwmvalue = *(PWMsort[PWMstate -1]);

        return;
    }

    use = PWMstate -1;
    hlp = use;

    while(pwmvalue >= *(PWMsort[use]) ){
        pwmpinoff((uint8_t*)(PWMsort[use]));
        use++;
        hlp ++;

        if(hlp == PWMchannels){
            break;
        }
    }

    if(use == PWMchannels){
        PwmTimerCountRegister = (uint8_t)(uint16_t) (0xFF - (0xFF - *(PWMsort[use-1])));
        PWMstate = 0;
        return;
    }

    PwmTimerCountRegister = (uint8_t)(uint16_t) (0xFF - (*(PWMsort[use]) - pwmvalue));

    pwmvalue = *(PWMsort[use]);
    PWMstate = use+1;
}


void startPwmCycle(void){ //starts new cycle (initializes variables and sets pins high)
    uint8_t hlp = 0;

    pwmvalue = 0;
    PWMstate = 1;

    for(hlp = 0 ; hlp < PWMchannels ; hlp++){
        if(PWM[hlp] != PwmLoad[hlp]){
            PWM[hlp] = PwmLoad[hlp];
        }
    }

    hlp = 0;

    hlp = organizepwm((uint8_t *) PWM, (uint8_t **) PWMsort); //returns number of channels being zero

    if(hlp > 0){
        PWMstate = hlp+1;
    }

    SetActivePwmChannels();
}

uint8_t organizepwm(uint8_t * pw, uint8_t ** point){ //Uses Bubblesort to sort PWM
    uint8_t         i = 0, c = 2, *tmp;

    for(i = 0 ; i < PWMchannels ; i++){
        point[i] = &(pw[i]);
    }

    while(c > 0){

        c = 0;

        for(i = 0 ; i < PWMchannels -1 ; i++){

            if( (*(point[i])) > (*(point[i+1])) ){
                tmp = point[i+1];
                point[i+1] = point[i];
                point[i] = tmp;

                c++;
            }
        }
    }

    c = 0;

    for(i = 0 ; i < PWMchannels ; i++){
        if(*(point[i]) == 0){
            c++;
        }
    }

    return c;

}

void SetActivePwmChannels(void){ //Turns matching pins high

    uint8_t i = 0;

    for(i = 0 ; i < PWMchannels ; i++){
        if(PWM[i] != 0 ){
            *(PWMports[i]) |= (1<<(PWMpins[i]));
        }
    }
    return;
}

void pwmpinoff(uint8_t * pin){ //Turns a PWM pin off.
    uint8_t i = 0;

    for(i = 0 ; i < PWMchannels ; i++){
        if(pin == &PWM[i]){
            *(PWMports[i]) &= ~(1<<(PWMpins[i]));
        }
    }
}

