
/*
-------------------------------------------------------
    Interface - A simple hardware abstraction library

    by Tobias Schlegel - www.tobias-schlegel.de
-------------------------------------------------------

Copyright (C) 2010  Tobias Schlegel,
                    tobias@drschlegel.de
                    www.tobias-schlegel.de

Report bugs to:     tobias@drschlegel.de

This program is free software; you can redistribute it and/or modify it under the terms of the GNU
General Public License as published by the Free Software Foundation; either version 3 of the License,
or any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/

/*
This is an excample of a standard driver.
*/

#include <avr/io.h>

#include "../../libinterface.h"

#include "twi_driver.h"
#include "twilib.h"

/**
    Write function...
    @param[in]      adr     Address offset for interface (can be fixed, depends on IF-config)
    @param[in,out]  * data  Pointer to data to write. can be used as two-way communication
    @param[in]      len     Amount of units to write (in n * if_datatype)
*/
uint8_t twi_write(uint16_t address, if_datatype * data, uint8_t len){

    uint8_t     adr = 0;

    adr = (uint8_t) address & 0xFF;

    MT(adr, data, len);

    return if_OK;
}


/**
    Read function...
    @param[in]      adr     Address offset for interface (can be fixed, depends on IF-config)
    @param[in,out]  * data  Pointer to data to read. can be used as two-way communication
    @param[in]      len     Amount of units to write (in n * if_datatype)
*/
uint8_t twi_read(uint16_t address, if_datatype * data, uint8_t len){

    uint8_t     adr = 0, txbytes = 0;

    adr = (uint8_t) address & 0xFF;
    txbytes = (uint8_t) ( (address & 0xFF00) >> 8);

    if(txbytes != 0){
        MT(adr, data, txbytes);
    }

    MR(adr, data, len);

    return if_OK;
}


/**
    Control function...
    @param[in]      control Control-Word
    @param[in,out]  * data  Pointer to data to read/write. can be used as two-way communication
*/
uint8_t twi_control(uint16_t control, if_datatype * data){

    switch(control){

        case twic_shutdown:
        break;

        case twic_init:
            init_Master();
        break;

        //case :
        //break;

        default:
            return if_CONTROL_UNKNOWN;
        break;
    }

    return if_OK;
}

