
/*
-------------------------------------------------------
    Interface - A simple hardware abstraction library

    by Tobias Schlegel - www.tobias-schlegel.de
-------------------------------------------------------

Copyright (C) 2010  Tobias Schlegel,
                    tobias@drschlegel.de
                    www.tobias-schlegel.de

Report bugs to:     tobias@drschlegel.de

This program is free software; you can redistribute it and/or modify it under the terms of the GNU
General Public License as published by the Free Software Foundation; either version 3 of the License,
or any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/

/*
This is an excample of a standard driver.
*/

#include <avr/io.h>
#include <avr/interrupt.h>

#include "../../libinterface.h"
#include "uart_driver.h"

#include "uart.h"

volatile uint8_t    fifo[uart_fifo_size], *fifoW, *fifoR, fifoFill = 0;
volatile uint8_t    status = if_OK, FifoStatus = 0;

/**
    Write function...
    @param[in]      adr     Address offset for interface (can be fixed, depends on IF-config)
    @param[in,out]  * data  Pointer to data to write. can be used as two-way communication
    @param[in]      len     Amount of units to write (in n * if_datatype)
*/
uint8_t uart_write(uint16_t address, if_datatype * data, uint8_t len){

    /*
    uint8_t     i = 0;


    for(i = 0 ; i < len ; i++){
        uart_putc(data[i]);
    }
    */

    uart_put_array(data, len);

    return if_OK;
}


/**
    Read function...
    @param[in]      adr     Address offset for interface (can be fixed, depends on IF-config)
    @param[in,out]  * data  Pointer to data to read. can be used as two-way communication
    @param[in]      len     Amount of units to write (in n * if_datatype)
*/
uint8_t uart_read(uint16_t address, if_datatype * data, uint8_t len){

    uint8_t i = 0;

    for(i = 0 ; i < len ; i++){

        if(fifoFill == 0){
            data[i] = 0;
            return uartw_RequestUnderrun;
        }

        if(status != if_OK){
            status = if_OK;
            return status;
        }

        if(fifoR > &(fifo[uart_fifo_size -1]) ){
            fifoR = fifo;
            FifoStatus &= ~(1<<0);
        }

        if((fifoR < fifoW) || ((fifoR > fifoW) && (FifoStatus & 0x01)) ){

            data[i] = *fifoR;
            fifoR++;
            fifoFill--;

        }
    }

    return if_OK;
}


/**
    Control function...
    @param[in]      control Control-Word
    @param[in,out]  * data  Pointer to data to read/write. can be used as two-way communication
*/
uint8_t uart_control(uint16_t control, if_datatype * data){

    char cnewl[] = {(char)13, (char) 10};

    switch(control){

        case uartc_shutdown:

        break;

        case uartc_init:
            init_uart( (uint16_t) ( (data[1] << 8) | data[0] ) );
            fifoW = (volatile uint8_t *) &fifo;
            fifoR = (volatile uint8_t *) &fifo;
            UCSR0B |= (1<<7);
        break;

        //---

        case uartc_newline:
            uart_put_array((uint8_t*) cnewl, 2);
        break;

        default:
            return if_CONTROL_UNKNOWN;
        break;
    }

    return if_OK;
}



ISR(USART0_RX_vect){

    if( fifoW > &(fifo[uart_fifo_size -1])){
        fifoW = fifo;
        FifoStatus |= (1<<0);
    }

    *fifoW = uart_getc();
    fifoFill++;

    fifoW++;

    if(fifoW == fifoR){
        fifoR = fifoW;
        status = uarte_IO_underrun;
    }

}
