/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: dynwatches.c
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include "dynwatches.h"
#include "ezmalloc.h"

#include "debug.h"

volatile watchhandle   *watches;
volatile uint8_t        watchcount = 0;

void dump(watchhandle W);

watchhandle new_watch(uint8_t prescaler){

    watchhandle neww, *tmp;
    neww = (watchhandle) malloc(sizeof(struct watch)); //reserve 4 byte for new watch

    if(!neww){
        return (watchhandle)0;
    }

    neww->pre = prescaler;

    watchcount ++;

    if(watchcount == 1){
        tmp = (watchhandle *) malloc(watchcount * sizeof(watchhandle));
    }else{
        tmp = (watchhandle *) calloc((void *)watches, (watchcount * sizeof(watchhandle)));
    }

    if(tmp == 0){
        delete((void *) neww);
        watchcount --;
        return (watchhandle)0;
    }

    watches = tmp;

    watches[watchcount -1] = neww;

    return neww;
}



void w_reset_watch(watchhandle watch){   //reset a watch

    if(watch == 0){
        return;
    }

    watch->p = 0;
    watch->w = 0;
}

uint16_t get_watch(watchhandle watch){

    if(watch == 0){
        return 0;
    }

    return watch->w;
}

uint8_t delete_watch(watchhandle watch){

    volatile watchhandle *tmp;

    if(watch == 0){
        return 1;
    }

    delete((void *) watch);

    watchcount--;

    tmp = (volatile watchhandle *) calloc((void *) watches, (watchcount * sizeof(watchhandle)));

    if(!tmp){
        return 2;
    }

    watches = tmp;

    return 0;
}

uint16_t * get_watch_pointer(watchhandle watch){

    uint8_t i = 0;

    for(i = 0 ; i < watchcount ; i++){
        if(watches[i] == watch){
            return (uint16_t *) &(watch->w);
        }
    }

    return (uint16_t *) 0;
}

void w_tick_watch(void){
    uint8_t     i = 0;


    for(i = 0 ; i < watchcount ; i++){

        watches[i]->p++;

        if(watches[i]->p == watches[i]->pre){
            watches[i]->w++;
            watches[i]->p = 0;
        }
    }

}
