/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: events.c
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <avr/io.h>

#include "events.h"


volatile event 		events[ev_event_size];

uint8_t		first_usable_event = 0;


void ev_init_events(void){

	uint8_t 	i = 0;

	for(i = 0 ; i < ev_event_size ; i++){

		events[i].status = ev_inactive;
	}

}


void ev_delete_event(uint8_t event){

	uint8_t 				i = 0;

	for(i = 0 ; i < ev_event_size ; i++){

		if(events[i].trigger == event){

			events[i].status = ev_inactive;
		}

	}
}


void ev_delete_pid(uint8_t uid){

	uint8_t 				i = 0;

	for(i = 0 ; i < ev_event_size ; i++){

		if(events[i].uid == uid){

			events[i].status = ev_inactive;
		}

	}
}

void ev_pause_event(uint8_t event){

    uint8_t 				i = 0;

    for(i = 0 ; i < ev_event_size ; i++){

		if(events[i].trigger == event){

			events[i].status = ev_dormant;
		}

	}
}


void ev_unpause_event(uint8_t event){

    uint8_t 				i = 0;

    for(i = 0 ; i < ev_event_size ; i++){

		if(events[i].trigger == event){

			events[i].status = ev_active;
		}

	}
}


void ev_pause_uid(uint8_t uid){

    uint8_t 				i = 0;

    for(i = 0 ; i < ev_event_size ; i++){

		if(events[i].uid == uid){

			events[i].status = ev_dormant;
		}

	}
}


void ev_unpause_uid(uint8_t uid){

    uint8_t 				i = 0;

    for(i = 0 ; i < ev_event_size ; i++){

		if(events[i].uid == uid){

			events[i].status = ev_active;
		}

	}
}


void ev_new_event(uint8_t uid, uint8_t event, EventCall Call){

	uint8_t 	i = 0;

	for(i = 0 ; i < ev_event_size ; i++){
		if(events[i].status == ev_inactive){
			events[i].status = ev_active;
			events[i].uid = uid;
			events[i].trigger = event;
			events[i].CallMe = Call;
			break;
		}
	}

	first_usable_event++;
}


uint8_t ev_count_active_events(void){		//count all active events
	uint8_t 	i = 0;

	for(i = 0 ; i < ev_event_size ; i++){

		if(events[i].status != ev_inactive){
			i++;
		}
	}

	return i;

}

#if ev_use_value_as_CallMe_parameters
void ev_event(uint8_t ev, void * p){		//Processes all eventointments
	uint8_t 	i = 0;

	for(i = 0 ; i < ev_event_size ; i++){

		if(events[i].status == ev_active){

			if(events[i].trigger == ev){
				(*(events[i].CallMe))(ev, p);
			}
		}
	}
}

#else

void ev_event(uint8_t ev){		//Processes all eventointments
	uint8_t 	i = 0;

	for(i = 0 ; i < ev_event_size ; i++){

		if(events[i].status == ev_active){

			if(events[i].trigger == ev){
				(*events[i].CallMe)();
			}
		}
	}
}

#endif

