
/*
-------------------------------------------------------
    Event manager Library Version 0.1
    An easy way to handle events.

    by Tobias Schlegel - www.tobias-schlegel.de

    Hardware: Any AVR with RAM
-------------------------------------------------------

Copyright (C) 2009  Tobias Schlegel,
                    tobias@drschlegel.de
                    www.tobias-schlegel.de

This program is free software; you can redistribute it and/or modify it under the terms of the GNU
General Public License as published by the Free Software Foundation; either version 3 of the License,
or any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program; if not, see <http://www.gnu.org/licenses/>.
*/


#include <avr/io.h>


#define ev_use_value_as_CallMe_parameters 1
#define ev_event_size 50


#define ev_inactive 0
#define ev_active   1
#define ev_dormant  2

#if ev_use_value_as_CallMe_parameters
typedef void (*EventCall) ( uint8_t, void * );           //Define a pointer-to-a-function globally
#else
typedef void (*EventCall) ( void );           //Define a pointer-to-a-function globally
#endif

struct Sevent{

    uint8_t						uid;
    uint8_t                     status;
    uint8_t                     trigger;
    EventCall                   CallMe;
};

typedef struct Sevent event;

extern volatile event events[ev_event_size];



//prototypes

void ev_init_events(void);

void ev_new_event(uint8_t uid, uint8_t event, EventCall Call);

#if ev_use_value_as_CallMe_parameters
void ev_event(uint8_t ev, void * p);
#else
void ev_event(uint8_t ev);
#endif

uint8_t ev_count_active_events(void);
void ev_delete_event_uid(uint8_t);

void ev_pause_event(uint8_t event);
void ev_unpause_event(uint8_t event);
void ev_pause_uid(uint8_t uid);
void ev_unpause_uid(uint8_t uid);
