
#include <avr/io.h>
#include <avr/interrupt.h>

#include "init.h"

#include "namesaliases.h"

#include "libinterface.h"
#include "driver\lcd\lcd_driver.h"
/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: init.c
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include "driver\max6675\max_driver.h"
#include "driver\uart\uart_driver.h"
#include "driver\twi\twi_driver.h"
#include "driver/pwm/PWMdriver.h"

#include "ui.h"
#include "events.h"
#include "scheduler.h"
#include "dynwatches.h"
#include "ezmalloc.h"
#include "displaymanagement.h"
#include "system.h"

void init_ints(void){

    //INT0
    //INT1
    EICRA = ( (1<<3) | (1<<0) );
    //EICRA |= ( (1<<0) );
    EIMSK = 0x03;

}

void init_timers(void){

    //TIMER 0
    TCNT0 = 0;
    TCCR0B = ((1<<2) | (1<<3)); //clk / 1024
    TIMSK0 = (1<<0);

    //TIMER 2
    TCNT1 = 62411;
    TCCR1B = 3;
    TIMSK1 = (1<<0);
}

void connects(void){

    uint8_t     data[3];

    if_createInterface(d_lcd, if_ACTIVE | if_WRITEONLY , 0, if_NO_TARGET, 0, lcd_write, if_NO_READ, lcd_control);
    if_createInterface(d_uart, if_ACTIVE, 0, if_NO_TARGET, 0, uart_write, uart_read, uart_control);
    if_createInterface(d_max, if_ACTIVE | if_READONLY , 100, if_NO_TARGET, 0, if_NO_WRITE, max_read, max_control);
    if_createInterface(d_pwm, if_ACTIVE, 0, if_NO_TARGET, 0, pwm_write, pwm_read, pwm_control);
    if_createInterface(d_twi, if_ACTIVE, 0, if_NO_TARGET, 0, twi_write, twi_read, twi_control);

    data[0] = lcda_on;
    ifControl(d_lcd, lcdc_init, data);

    //data[0] = 64; //19200 baud
    data[0] = 10; //115200 baud
    data[1] = 0;
    ifControl(d_uart, 1, data);

    ifControl(d_max, 1, data);
    ifControl(d_pwm, 1, data);

    sh_new_appointment(sh_app_resident | sh_app_buffered, shui_displaymanagement, dm_refreshLcd, sys_lcd_refresh_interval, sys_lcd_refresh_interval);
    sh_new_appointment(sh_app_resident, sys_measuring, sys_measure, sys_measuring_interval, sys_measuring_interval);
    sh_new_appointment(sh_app_resident, sys_safe, sys_safety, sys_safety_interval, sys_safety_interval);

    ev_new_event(1, evt_NewMeasurements, regulate);
}

void init_avr(void){

    //(1<<) |
    DDRA = ( 0);
    DDRB = ( (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<6) );
    DDRC = ( (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7) );
    DDRD = ( (1<<7) );

    PORTA = 0;
    PORTB = 0;
    PORTC = 0;
    PORTD = 28;

    init_ui();

    init_ints();
    init_timers();
    init_memory();

    ev_init_events();
    sh_init_horrario();
    //init_watches();

    if_init();

    connects();

    sei();
}



