/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: libInput.c
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <inttypes.h>

#include "libInput.h"
#include "string.h"

#include "debug.h"


uint8_t inRlg10(uint16_t n){
    uint8_t i = 0;
    uint16_t div = 1;

    if(n < 10){
        return 0;
    }

    for(i = 1 ; i < 255 ; i++){
        div *= 10;
        if((n / div) < 10){
            return i;
        }
    }

    return i;
}


uint8_t inUI16(char *cOut, uint8_t * len, char * cTitle, char * cUnit, uint8_t key, uint16_t * value, uint8_t fraction, uint8_t step, uint8_t faststep){

    char cNewLine[2] = {(char)10, (char) 0}, cSymbol[2] = {(char) '~', (char) 0}, cNumbers[21];
    uint16_t hilf = 0, val = 0;
    uint8_t NumLen = 0, ret = inR_NOP;

    switch(key){
        case inKey_abort:
            ret = inR_Aborted;
        break;

        case inKey_OK:
            ret = inR_Complete;
        break;

        case inKey_minus:
            (*value) -= step;
            cSymbol[0] = '-';
        break;

        case inKey_plus:
            (*value) += step;
            cSymbol[0] = '+';
        break;

        case inKey_minusFast:
            (*value) -= faststep;
            cSymbol[0] = '<';
        break;

        case inKey_plusFast:
            (*value) += faststep;
            cSymbol[0] = '>';
        break;
    }

    if(cOut != 0){
        val = (*value);
        (*len) = 0;

        str_cpy(cOut, cTitle);
        str_cat(cOut, cNewLine);
        str_cat(cOut, cSymbol);
        str_cat_p(cOut, " ");

        if(fraction != 0){

            hilf = val / fraction;
            NumLen = inRlg10(hilf);
            NumLen += 1;
            inttostr16(cNumbers, hilf, NumLen);

            str_cat(cOut, cNumbers);
            (*len) += str_len(cNumbers);

            str_cat_p(cOut, ".");

            hilf = val % fraction;
            hilf *= 1000;
            hilf /= fraction;
            NumLen = inRlg10(hilf);
            NumLen += 1;
            inttostr16(cNumbers, hilf, NumLen);

            str_cat(cOut, cNumbers);
            (*len) += str_len(cNumbers);

        }else{
            NumLen = inRlg10(val);
            inttostr16(cNumbers, val, NumLen);

            str_cat(cOut, cNumbers);
            (*len) += str_len(cNumbers);
        }

        str_cat(cOut, cUnit);

        (*len) += str_len(cTitle);
        (*len) += str_len(cNewLine);
        (*len) += str_len(cSymbol);
        (*len) += str_len(cUnit);
        (*len) += 2;
    }

    return ret;
}


uint8_t inI8(char *cOut, uint8_t * len, char * cTitle, char * cUnit, uint8_t key, int8_t * value, uint8_t fraction, uint8_t step, uint8_t faststep){

    char cNewLine[2] = {(char)10, (char) 0}, cSymbol[2] = {(char) '~', (char) 0}, cNumbers[21];
    int8_t hilf = 0, val = 0;
    uint8_t NumLen = 0, ret = inR_NOP;

    switch(key){
        case inKey_abort:
            ret = inR_Aborted;
        break;

        case inKey_OK:
            ret = inR_Complete;
        break;

        case inKey_minus:
            (*value) -= step;
            cSymbol[0] = '-';
        break;

        case inKey_plus:
            (*value) += step;
            cSymbol[0] = '+';
        break;

        case inKey_minusFast:
            (*value) -= faststep;
            cSymbol[0] = '<';
        break;

        case inKey_plusFast:
            (*value) += faststep;
            cSymbol[0] = '>';
        break;
    }

    if(cOut != 0){
        val = (*value);
        (*len) = 0;

        str_cpy(cOut, cTitle);
        str_cat(cOut, cNewLine);
        str_cat(cOut, cSymbol);
        str_cat_p(cOut, " ");

        if((*value) < 0){
            str_cat_p(cOut, "-");
        }else{
            str_cat_p(cOut, "+");
        }

        if(fraction != 0){

            hilf = val / fraction;
            NumLen = inRlg10(hilf);
            NumLen += 1;
            inttostr8(cNumbers, hilf, NumLen);

            str_cat(cOut, cNumbers);
            (*len) += str_len(cNumbers);

            str_cat_p(cOut, ".");

            hilf = val % fraction;
            hilf *= 1000;
            hilf /= fraction;
            NumLen = inRlg10(hilf);
            NumLen += 1;
            inttostr8(cNumbers, hilf, NumLen);

            str_cat(cOut, cNumbers);
            (*len) += str_len(cNumbers);

        }else{
            NumLen = inRlg10(val);
            inttostr8(cNumbers, val, NumLen);

            str_cat(cOut, cNumbers);
            (*len) += str_len(cNumbers);
        }

        str_cat(cOut, cUnit);

        (*len) += str_len(cTitle);
        (*len) += str_len(cNewLine);
        (*len) += str_len(cSymbol);
        (*len) += str_len(cUnit);
        (*len) += 2;
    }

    return ret;
}

