
/*
*   LibInterface
*   A very basic hardware abstraction library.
*   -> Main header file
*
*   File: libinterface.h
*
*   Version:    1.0
*   File-Ver.:  1.0 (05.Jan.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       5.Jan.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <inttypes.h>

#include "libinterface_types.h"

//Table size must be smaller than 255
#define if_INTERFACE_TABLE_SIZE      30

//set this to 1 to allow write to multiple interfaces using only one call
#define if_ALLOW_MULTIPLE_WRITE      0

uint8_t     if_init(void);

uint8_t     if_createInterface(uint8_t id, uint8_t config, uint16_t address, if_datatype * target, uint8_t target_size, if_write wr, if_read rd, if_control ctl);
uint8_t     if_deleteInterface(uint8_t id);
uint8_t     if_lockInterface(uint8_t id);
uint8_t     if_unlockInterface(uint8_t id);

uint8_t     ifWrite(uint8_t id, uint16_t adr, if_datatype * data, uint8_t len);
uint8_t     ifRead(uint8_t id, uint16_t adr, if_datatype * data, uint8_t len);
uint8_t     ifControl(uint8_t id, uint16_t ctl, if_datatype * data);

/**
    ZERO placeholders
*/
#define     if_NO_TARGET                ((if_datatype *) 0)
#define     if_NO_DATA                  ((if_datatype *) 0)
#define     if_NO_WRITE                 ((if_write) 0)
#define     if_NO_READ                  ((if_read) 0)
#define     if_NO_CONTROL               ((if_control) 0)


/**
    Configuraton possibilities (bits)
*/
#define     if_UNUSED                   0
#define     if_ACTIVE                   (1<<0)
#define     if_LOCKED                   (1<<1)
#define     if_READONLY                 (1<<2)
#define     if_WRITEONLY                (1<<3)
#define     if_MEMORY_TARGET            (1<<4)
#define     if_FIXED_ADDRESS            (1<<5)
#define     if_READ_FROM_MULTI          (1<<6)
#define     if_ONLINE                   (1<<7)


/**
    Errorcodes
*/
#define     if_OK                       0       /// Everything's all right
#define     if_NO_CONTROL_TARGET        1       /// No control function specified
#define     if_NO_WRITE_TARGET          2       /// No write function specified
#define     if_NO_READ_TARGET           3       /// No read function specified
#define     if_NO_MEMORY_TARGET         4       /// No memory target speified
#define     if_DRIVER_FUNCTION_ERROR    5       /// Error in driver function (check data)
#define     if_IF_LOCKED                6       /// The interface is in locked state
#define     if_BAD_CONFIG               7       /// The interface-configuration is erronous
#define     if_IF_TABLE_FULL            8       /// No space for new interfaces
#define     if_NOT_FOUND                9       /// The specified interface-ID was not found - 404 sorry
#define     if_READ_ONLY                10      /// The IF is specified readonly thus no writing.
#define     if_WRITE_ONLY               11      /// The IF is specified writeonly thus no reading.
#define     if_NO_MULTIWRITE            12      /// Not sure what this was..
#define     if_MULTIPLE_CREATE          13      /// The ID already exists and Multiwrite is not enabled
#define     if_TARGET_SIZE_ZERO         14      /// The target size to write to is zero
#define     if_WRITE_SIZE_LIMIT         15      /// The write request exceeds maximum write space
#define     if_OFFLINE                  16      /// The interface is offline
#define     if_CONTROL_UNKNOWN          17      /// The control word used is unknown or not supported
#define     if_ERROR                    18      /// Just another "error"

#define     if_SHOOT_ME                 255 //In windows this is known as "you shouldn't be able to see this"


