
/*
*   MenuLib
*   A library for generating simple tree based menus on small (embedded) systems.
*   Download a demo application from www.tobias-schlegel.de
*
*   File: menu.h
*
*   Version:    1.0 (2.Jan.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       2.Jan.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <inttypes.h>

#define cMenuLine 16 //Maximum length of a menu entry
#define mCallbackWithParams 1 //Use Callback functions with a (preshared) parameter

//Undefine to deactivate feature
#define mOffsetString "+" //A char (string) to display a father-> child relation
#define mOffsetShortenedString "#" //At some point the tree history must be shortened. Symbos for this. Undefine only in combination with mOffsetString
#define mMarkedString ">" //Char (string) to mark the selected item
#define mNotMarkedString "-" //Char (string) to mark not selected item

#define mKeyUp      2 //Keycode for "up"
#define mKeyDown    1 //Keycode for "down"
#define mKeyOK      0x81 //Keycode for "OK" (deeper)
#define mKeyBack    0x14 //Keycode for "back"

//Do not edit below.

#ifndef __menu_type
#define __menu_type 1

#ifdef mCallbackWithParams

typedef void (*menucallback)(uint8_t);

typedef struct {
    menucallback CallBack;
    uint8_t CallBackArgument;
    char cText[cMenuLine];
    uint8_t children;
}menudata;
#else

typedef void (*menucallback)(void);

typedef struct {
    menucallback CallBack;
    char cText[cMenuLine];
    uint8_t children;
}menudata;
#endif

#define menuCnull (menucallback) 0  //placeholder for "nothing"

#endif

void init_menu(menudata * menu, uint8_t entries);
void mGetText(char * txt, uint8_t lines, uint8_t linelen, uint8_t parents);
void mStatemachine(uint8_t key);


