/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: pid.c
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <inttypes.h>

#include "pid.h"

#include "debug.h"

void setSetpoint(PIDdata * piddata, int16_t setpnt){
    piddata->INT = 0;
    piddata->lastE = 0;

    piddata->setpoint = setpnt;
}

void ResetPID(PIDdata * piddata){
    piddata->INT = 0;
    piddata->lastE = 0;
}

uint8_t MkPID(PIDdata * piddata, int16_t Kp, int16_t Ki, int16_t Kd, int16_t setpnt){
    piddata->INT = 0;
    piddata->lastE = 0;

    piddata->pPart = Kp;
    piddata->iPart = Ki;
    piddata->dPart = Kd;

    piddata->setpoint = setpnt;

    return 0;
}

int32_t PID(PIDdata* piddata, int16_t is){
    int16_t e = 0, dt = 0;
    int32_t p = 0, i = 0, d = 0;
    int32_t pid = 0;

    //char cout[261], cNumbers[26];

    e = piddata->setpoint - is;
    dt = e - piddata->lastE;

    piddata->lastE = e;
    piddata->INT += e;

    if((piddata->pPart) != 0){
        p = (int32_t)(piddata->pPart);
        p *= e;
        p /= 100;
    }

    if((piddata->iPart) != 0){
        i = (piddata->iPart);
        i *= piddata->INT;
        i /= 100;
    }

    if((piddata->dPart) != 0){
        d = (piddata->dPart);
        d *= dt;
        d /= 100;
    }

    pid = p;
    pid += i;
    pid += d;

    return pid;
}
