/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: scheduler.h
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <avr/io.h>

#define sh_use_value_and_uid_as_CallMe_parameters 0
#define sh_horrario_size 20




#define sh_app_inactive 0
#define sh_app_resident 1
#define sh_app_once 2

#define sh_app_buffered (1<<6)
#define sh_app_buff_trig (1<<7)

#define sh_type_mask 0x3F

#if sh_use_value_and_uid_as_CallMe_parameters
typedef void (*SchedCall) ( uint8_t, uint16_t );           //Define a pointer-to-a-function globally
#else
typedef void (*SchedCall) ( void );           //Define a pointer-to-a-function globally
#endif

struct appointment{

    uint8_t						uid;

	volatile uint8_t 			type;

	volatile uint16_t			time;
	uint16_t					pre_time;

    SchedCall                   CallMe;
    #if sh_use_value_and_uid_as_CallMe_parameters
    uint16_t                    value;
    #endif
};

typedef struct appointment app;

extern volatile app horrario[sh_horrario_size];



//prototypes

void sh_init_horrario(void);

#if sh_use_value_and_uid_as_CallMe_parameters
void sh_new_appointment(uint8_t type, uint8_t uid, SchedCall Call, uint16_t value, uint16_t time, uint16_t preset);
#else
void sh_new_appointment(uint8_t type, uint8_t uid, SchedCall Call, uint16_t time, uint16_t preset);
#endif

void sh_process_app(volatile app *);
void sh_timer(void);
void sh_buffered(void);
uint8_t sh_count_active_apps(void);
void sh_delete_app_uid(uint8_t);

uint8_t sh_search_resident_uid_id(uint8_t, uint8_t);
uint8_t sh_search_uid_id(uint8_t, uint8_t);
uint8_t sh_search_type_id(uint8_t, uint8_t);
uint8_t sh_get_uid(uint8_t);
uint8_t sh_get_type(uint8_t);
#if sh_use_value_and_uid_as_CallMe_parameters
uint16_t sh_get_value(uint8_t);
#endif
uint16_t sh_get_time(uint8_t);
uint16_t sh_get_pre_time(uint8_t);
