/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: sheduler.c
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/


#include <avr/io.h>

#include "scheduler.h"


volatile app 		horrario[sh_horrario_size];

uint8_t		first_usable_app = 0;


void sh_init_horrario(void){

	uint8_t 	i = 0;

	for(i = 0 ; i < sh_horrario_size ; i++){

		horrario[i].type = sh_app_inactive;

	}

}


void sh_delete_app_uid(uint8_t uid){

	uint8_t 				i = 0;

	for(i = 0 ; i < sh_horrario_size ; i++){

		if(horrario[i].uid == uid){

			horrario[i].type = sh_app_inactive;

		}

	}
}

#if sh_use_value_and_uid_as_CallMe_parameters

void sh_new_appointment(uint8_t type, uint8_t uid, SchedCall Call, uint16_t value, uint16_t time, uint16_t preset){

	uint8_t 	i = 0;

	for(i = 0 ; i < sh_horrario_size ; i++){
		if(horrario[i].type == sh_app_inactive){
			horrario[i].type = type;
			horrario[i].uid = uid;
			horrario[i].time = time;
			horrario[i].pre_time = preset;
			horrario[i].CallMe = Call;
			horrario[i].value = value;
			break;
		}
	}

	first_usable_app++;
}

#else

void sh_new_appointment(uint8_t type, uint8_t uid, SchedCall Call, uint16_t time, uint16_t preset){

	uint8_t 	i = 0;

	for(i = 0 ; i < sh_horrario_size ; i++){
		if(horrario[i].type == sh_app_inactive){
			horrario[i].type = type;
			horrario[i].uid = uid;
			horrario[i].time = time;
			horrario[i].pre_time = preset;
			horrario[i].CallMe = Call;
			break;
		}
	}

	first_usable_app++;
}

#endif




uint8_t sh_search_uid_id(uint8_t uid, uint8_t start){


	uint8_t 	i = 0;

	for(i = start ; i < sh_horrario_size ; i++){
		if(horrario[i].uid == uid){

			return i;
		}
	}

	return i+1;
}

uint8_t sh_search_resident_uid_id(uint8_t uid, uint8_t start){


	uint8_t 	i = 0;

	for(i = start ; i < sh_horrario_size ; i++){
		if((horrario[i].uid == uid) && (horrario[i].type == sh_app_resident)){

			return i;
		}
	}

	return i+1;
}


uint8_t sh_search_type_id(uint8_t type, uint8_t start){


	uint8_t 	i = 0;

	for(i = start ; i < sh_horrario_size ; i++){
		if(horrario[i].type == type){

			return i;
		}
	}

	return i+1;
}


uint8_t sh_get_uid(uint8_t id){

	return horrario[id].uid;
}

uint8_t sh_get_type(uint8_t id){

	return horrario[id].type;
}

#if sh_use_value_and_uid_as_CallMe_parameters
uint16_t sh_get_value(uint8_t id){

	return horrario[id].value;
}
#endif

uint16_t sh_get_time(uint8_t id){

	return horrario[id].time;
}

uint16_t sh_get_pre_time(uint8_t id){

	return horrario[id].pre_time;
}

uint8_t sh_count_active_apps(void){		//count all active apps
	uint8_t 	i = 0;

	for(i = 0 ; i < sh_horrario_size ; i++){


		if(horrario[i].type != sh_app_inactive){

			i++;

		}

	}

	return i;

}


void sh_timer(void){		//Processes all appointments
	uint8_t 	i = 0, t = 0;

	for(i = 0 ; i < sh_horrario_size ; i++){

		if(horrario[i].type != sh_app_inactive){

			horrario[i].time --;

			if(horrario[i].time == 0){
				t = (horrario[i].type);

                t &= sh_type_mask;

                switch(t){

                    case sh_app_resident:		//is resident
                        horrario[i].time = horrario[i].pre_time;

                        if((horrario[i].type) & sh_app_buffered){
                            (horrario[i].type) |= sh_app_buff_trig;
                        }else{
                            #if sh_use_value_and_uid_as_CallMe_parameters
                            horrario[i].CallMe(horrario[i].uid, horrario[i].value);
                            #else
                            horrario[i].CallMe();
                            #endif
                        }
                        break;

                    case sh_app_once:		//once
                        if((horrario[i].type) & sh_app_buffered){
                            (horrario[i].type) |= sh_app_buff_trig;
                        }else{
                            horrario[i].type = sh_app_inactive;

                            #if sh_use_value_and_uid_as_CallMe_parameters
                            horrario[i].CallMe(horrario[i].uid, horrario[i].value);
                            #else
                            horrario[i].CallMe();
                            #endif
                        }
                        break;

                    default:								//else
                        horrario[i].type = sh_app_inactive;
                        break;

                }
			}
		}
	}

}

void sh_buffered(void){ //Process buffered apps

    	uint8_t 	i = 0, t = 0;

	for(i = 0 ; i < sh_horrario_size ; i++){

		if(horrario[i].type != sh_app_inactive){

			if(horrario[i].type & sh_app_buff_trig){
				t = (horrario[i].type);

                t &= sh_type_mask;

                switch(t){

                    case sh_app_resident:		//is resident

                        #if sh_use_value_and_uid_as_CallMe_parameters
                        horrario[i].CallMe(horrario[i].uid, horrario[i].value);
                        #else
                        horrario[i].CallMe();
                        #endif
                        break;

                    case sh_app_once:		//once
                        horrario[i].type = sh_app_inactive;

                        #if sh_use_value_and_uid_as_CallMe_parameters
                        horrario[i].CallMe(horrario[i].uid, horrario[i].value);
                        #else
                        horrario[i].CallMe();
                        #endif
                        break;

                    default:								//else
                        horrario[i].type = sh_app_inactive;
                        break;

                }

                horrario[i].type &= ~sh_app_buff_trig;

			}
		}
	}

}


