/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: system.c
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <avr/io.h>

#include "system.h"

#include "namesaliases.h"
#include "libinterface.h"
#include "events.h"
#include "settings.h"

#include "pid.h"

#include "driver/max6675/max_driver.h"

PIDdata regs[regulator_amount];
ThermoDescr Descriptors[regulator_amount] = {{0, 0x0100, 0, 2, &(regs[0])}, {1, 0x0101, 1, 2, &(regs[1])}};

void sys_measure(void){

    ///add measurements here

    ifControl(d_max, max_measure, if_TODTP 0);


    ev_event(evt_NewMeasurements, (void*) 0);
}

void sys_safety(void){
    uint8_t tc = 0, i = 0;
    uint16_t temp = 0;
    ///add periodic safety funcs here

    for(i = 0; i < regulator_amount ; i++){
        ifRead(d_max, Descriptors[i].therm_adr, if_TODTP &temp, 2);
        ifRead(d_max, Descriptors[i].TC_adr, if_TODTP &tc, 2);

        if( (tc == 0) && (temp < 900)){
            Descriptors[i].status = ThermErr_TcOpen;
        }

        if((tc != 0) && (Descriptors[i].status == ThermErr_TcOpen)){
            Descriptors[i].status = ThermErr_OffNoErr;
        }
    }
}

void sys_deactivate(uint8_t thermID){

    if(thermID >= regulator_amount){
        return;
    }

    if(Descriptors[thermID].status == 0){
        Descriptors[thermID].status = ThermErr_UserOff;
        ResetPID((Descriptors[thermID].regulator));
    }
}

void sys_init(void){
    uint8_t i = 0;

    for(i = 0 ; i < regulator_amount ; i++){
        MkPID(&(regs[i]), load_byte_setting(&(set_PidParams[i][0])), load_byte_setting(&(set_PidParams[i][1])), load_byte_setting(&(set_PidParams[i][2])), load_word_setting(&(set_HeatSetpoints[i])));
    }
}

void sys_activate(uint8_t thermID){
    uint8_t tc = 0;

    if(thermID >= regulator_amount){
        return;
    }

    ifRead(d_max, Descriptors[thermID].TC_adr, if_TODTP &tc, 2);
#warning Not tested patch in line below.
    if((Descriptors[thermID].status != ThermErr_TcOpen) && (tc != 0)){
        Descriptors[thermID].status = 0;
        ResetPID((Descriptors[thermID].regulator));
    }
}

void regulate(uint8_t trigger, void * in){
    uint16_t temp = 0;
    int32_t soll = 0;
    uint8_t pwm = 0;
    uint8_t i = 0;

    for(i = 0; i < regulator_amount ; i++){

        if(Descriptors[i].status == 0){

            ifRead(d_max, Descriptors[i].therm_adr, if_TODTP &temp, 2);

            soll = PID(Descriptors[i].regulator, temp);

            if(soll > 255){
                pwm = 255;
            }else if(soll >= 0){
                pwm = (uint8_t) soll;
            }else if(soll < 0){
                pwm = 0;
            }

            ifWrite(d_pwm, Descriptors[i].PWM_adr, if_TODTP &pwm, 1);
        }else{
            pwm = 0;
            ifWrite(d_pwm, Descriptors[i].PWM_adr, if_TODTP &pwm, 1);
        }
    }
}


