/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: system.h
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <inttypes.h>

#include "pid.h"
#include "namesaliases.h"

#define sys_measuring_interval  25
#define sys_lcd_refresh_interval    10
#define sys_safety_interval  25

#define ThermErr_TcOpen     1
#define ThermErr_UserOff    2
#define ThermErr_OffNoErr   3

#ifndef __ThermoDescr
#define __ThermoDescr 1
/// regulator descriptor
typedef struct _ThermoDescr{
    uint16_t    therm_adr;
    uint16_t    TC_adr;
    uint16_t    PWM_adr;
    uint8_t     status;
    PIDdata *   regulator;
}ThermoDescr;
#endif

extern ThermoDescr Descriptors[regulator_amount];

void sys_measure(void);
void sys_safety(void);
void sys_activate(uint8_t heatID);
void sys_deactivate(uint8_t heatID);
void sys_init(void);

void regulate(uint8_t trigger, void * in);

