/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: ui.c
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <avr/io.h>

#include "ui.h"
#include "scheduler.h"
#include "namesaliases.h"
#include "events.h"

#include "debug.h"


volatile int8_t         ui_rotations = 0;
volatile uint32_t       ui_uptime = 0;
volatile uint8_t        ui_KeyFifo[ui_KeyFifoSize], *KeyFifoW, *KeyFifoR, KeyFifoStatus = 2;
volatile uint8_t        ui_pending = 0;

void init_ui(void){
    DDRD &= ~( (1<<2) | (1<<3) | (1<<4) );

    KeyFifoR = ui_KeyFifo;
    KeyFifoW = ui_KeyFifo;
}

uint8_t ui_get_keystate(void){

    return ui_KeyFifoRead();
}

uint8_t ui_KeyFifoRead(void){
    uint8_t tmp = 0;

    if(KeyFifoR > &(ui_KeyFifo[ui_KeyFifoSize -1]) ){
        KeyFifoR = ui_KeyFifo;
        KeyFifoStatus &= ~(1<<0);
    }

    if((KeyFifoR < KeyFifoW) || ((KeyFifoR > KeyFifoW) && (KeyFifoStatus & 0x01)) ){

        tmp = *KeyFifoR;
        KeyFifoR++;

    }

    return tmp;
}

void ui_KeyIn(uint8_t key){
    uint8_t lastkey = 0;

    lastkey = *(KeyFifoW -1);

    switch(key){

        case 0x04:
        case 0x08:
            if(key == lastkey){
                return;
            }

            if((key == 0x08) && (ui_pending == uik_longdn)){
                ui_KeyFifoWrite(uik_longdn);
                ui_pending = 0;
                lastkey = uik_longdn;
            }

            if( (lastkey == 0x04) && (key == 0x08)){
                ui_KeyFifoWrite(0x08); // for completion
                key = uik_klick;
            }

        break;

        case 0x10:
            if(lastkey == 0x04){
               ui_pending = uik_longdn;
               sh_new_appointment(sh_app_once, shui_longlongkey, ui_KeyLongLong, ui_longlongtime, 0);
               return;
            }else{
                return;
            }
        break;

        case 0x20:
            if(lastkey == 0x04){
               key = uik_longlongdn;
               ui_pending = 0;
            }else{
                return;
            }
        break;
    }


    ui_KeyFifoWrite(key);

}

void ui_KeyFifoWrite(uint8_t key){

    if( KeyFifoW > &(ui_KeyFifo[ui_KeyFifoSize -1]) ){
        KeyFifoW = ui_KeyFifo;
        KeyFifoStatus |= (1<<0);
    }

    (*KeyFifoW) = key;
    KeyFifoW++;

    ev_event(evt_NewKey, &key);

}

void ui_KeyLong(void){
    ui_KeyIn(uik_mask_long);
}

void ui_KeyLongLong(void){
    ui_KeyIn(uik_mask_longlong);
}

void LedOn(void){
    PORTB |= (1<<6);
}


void LedOff(void){
    PORTB &= ~(1<<6);
}


void LedToggle(void){
    PORTB ^= (1<<6);
}

