/*
*   Hot air controller
*   Controller for a hot air soldering gun
*
*   File: ui.h
*
*   Version:    1.0 beta (06.03.2011)
*   Author:     Tobias Schlegel, DK7SCH
*   Web:        www.tobias-schlegel.de
*   Contact:    tobias@drschlegel.de
*   Date:       06.03.2011
*   License:    Creative Commons CC-BY 3.0
*               http://creativecommons.org/licenses/by/3.0/deed.en
*
*               Attribution:    Attribute with a copy of this code header and (if applicable)
*                               a notation in the "About" page of your product.
*
*/

#include <avr/io.h>

#include "dynwatches.h"

#define ui_KeyFifoSize      50

#define uik_no              0x00
#define uik_right           0x01
#define uik_left            0x02
#define uik_down            0x04
#define uik_dn_left         0x06
#define uik_dn_right        0x05
#define uik_up              0x08
#define uik_klick           0x81

#define uik_longdn_left     0x16
#define uik_longdn_right    0x15

#define uik_longdn          0x14
#define uik_longlongdn      0x24

#define uik_mask_long       0x10
#define uik_mask_longlong   0x20


#define ui_longtime     50 ///  time it needs to press DOWN to trigger "long" mask (in 10ms steps)
#define ui_longlongtime 250 ///  time it needs to press DOWN to trigger "longlong" mask (in 10ms steps)

extern volatile uint8_t     ui_keystate;
extern volatile int8_t      ui_rotations;
extern volatile uint32_t    ui_uptime;
extern volatile watchhandle    ui_watch;

void init_ui(void);
uint8_t ui_get_keystate(void);

uint8_t ui_KeyFifoRead(void);
void ui_KeyFifoWrite(uint8_t key);
void ui_KeyIn(uint8_t key);
void ui_KeyLong(void);
void ui_KeyLongLong(void);

void LedOn(void);
void LedOff(void);
void LedToggle(void);

